@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 2011-2016 Michael Goffioul
+Copyright (C) 2011-2018 Michael Goffioul
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -31,7 +31,7 @@ along with Octave; see the file COPYING.  If not, see
 namespace QtHandles
 {
 
-  FigureWindow::FigureWindow (QWidget* xparent)
+  FigureWindow::FigureWindow (QWidget *xparent)
     : FigureWindowBase (xparent)
   {
     // set icon from application resources
@@ -45,8 +45,7 @@ namespace QtHandles
   FigureWindow::createPopupMenu (void)
   {
     // For the time being, disable menubar/toolbar popup menu
-    return 0;
+    return nullptr;
   }
 
 }
-
