@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 2011-2017 Michael Goffioul
+Copyright (C) 2011-2018 Michael Goffioul
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -36,17 +36,17 @@ namespace QtHandles
   PopupMenuControl*
   PopupMenuControl::create (const graphics_object& go)
   {
-    Object* parent = Object::parentObject (go);
+    Object *parent = Object::parentObject (go);
 
     if (parent)
       {
-        Container* container = parent->innerContainer ();
+        Container *container = parent->innerContainer ();
 
         if (container)
           return new PopupMenuControl (go, new QComboBox (container));
       }
 
-    return 0;
+    return nullptr;
   }
 
   PopupMenuControl::PopupMenuControl (const graphics_object& go, QComboBox *box)
@@ -58,7 +58,7 @@ namespace QtHandles
 
     update (uicontrol::properties::ID_VALUE);
 
-    connect (box, SIGNAL (currentIndexChanged (int)),
+    connect (box, SIGNAL (activated (int)),
              SLOT (currentIndexChanged (int)));
   }
 
@@ -68,7 +68,7 @@ namespace QtHandles
   void PopupMenuControl::update (int pId)
   {
     uicontrol::properties& up = properties<uicontrol> ();
-    QComboBox* box = qWidget<QComboBox> ();
+    QComboBox *box = qWidget<QComboBox> ();
 
     switch (pId)
       {
@@ -80,7 +80,7 @@ namespace QtHandles
           box->clear ();
           box->addItems (Utils::fromStdString
                          (up.get_string_string ()).split ('|'));
-          if (box->count() > 0
+          if (box->count () > 0
               && oldCurrent >= 0
               && oldCurrent < box->count ())
             {
