@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 2011-2017 Michael Goffioul
+Copyright (C) 2011-2018 Michael Goffioul
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -37,21 +37,21 @@ namespace QtHandles
   PushButtonControl*
   PushButtonControl::create (const graphics_object& go)
   {
-    Object* parent = Object::parentObject (go);
+    Object *parent = Object::parentObject (go);
 
     if (parent)
       {
-        Container* container = parent->innerContainer ();
+        Container *container = parent->innerContainer ();
 
         if (container)
           return new PushButtonControl (go, new QPushButton (container));
       }
 
-    return 0;
+    return nullptr;
   }
 
   PushButtonControl::PushButtonControl (const graphics_object& go,
-                                        QPushButton* btn)
+                                        QPushButton *btn)
     : ButtonControl (go, btn)
   {
     btn->setAutoFillBackground (true);
@@ -64,7 +64,7 @@ namespace QtHandles
   PushButtonControl::update (int pId)
   {
     uicontrol::properties& up = properties<uicontrol> ();
-    QPushButton* btn = qWidget<QPushButton> ();
+    QPushButton *btn = qWidget<QPushButton> ();
 
     switch (pId)
       {
