@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 2011-2017 Michael Goffioul
+Copyright (C) 2011-2018 Michael Goffioul
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -38,28 +38,31 @@ namespace QtHandles
   SliderControl*
   SliderControl::create (const graphics_object& go)
   {
-    Object* parent = Object::parentObject (go);
+    Object *parent = Object::parentObject (go);
 
     if (parent)
       {
-        Container* container = parent->innerContainer ();
+        Container *container = parent->innerContainer ();
 
         if (container)
           return new SliderControl (go, new QScrollBar (container));
       }
 
-    return 0;
+    return nullptr;
   }
 
   SliderControl::SliderControl (const graphics_object& go,
-                                QAbstractSlider* slider)
+                                QAbstractSlider *slider)
     : BaseControl (go, slider), m_blockUpdates (false)
   {
     uicontrol::properties& up = properties<uicontrol> ();
 
     slider->setTracking (false);
     Matrix bb = up.get_boundingbox ();
-    slider->setOrientation (bb(2) > bb(3) ? Qt::Horizontal : Qt::Vertical);
+    bool vertical_slider = ( bb(2) < bb(3) );
+    slider->setOrientation (vertical_slider ? Qt::Vertical : Qt::Horizontal);
+    if (vertical_slider)
+      slider->setInvertedAppearance (true);  // Matlab compatibility
     Matrix steps = up.get_sliderstep ().matrix_value ();
     slider->setMinimum (0);
     slider->setMaximum (RANGE_INT_MAX);
@@ -84,7 +87,7 @@ namespace QtHandles
   SliderControl::update (int pId)
   {
     uicontrol::properties& up = properties<uicontrol> ();
-    QScrollBar* slider = qWidget<QScrollBar> ();
+    QScrollBar *slider = qWidget<QScrollBar> ();
 
     switch (pId)
       {
