@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 1996-2016 John W. Eaton
+Copyright (C) 1996-2018 John W. Eaton
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -38,6 +38,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "ov-fcn.h"
 #include "ov-cell.h"
 #include "pager.h"
+#include "parse.h"
 #include "unwind-prot.h"
 #include "utils.h"
 #include "variables.h"
@@ -77,7 +78,7 @@ dassl_user_function (const ColumnVector& x, const ColumnVector& xdot,
 
       try
         {
-          tmp = dassl_fcn->do_multi_index_op (1, args);
+          tmp = octave::feval (dassl_fcn, args, 1);
         }
       catch (octave::execution_exception& e)
         {
@@ -88,7 +89,7 @@ dassl_user_function (const ColumnVector& x, const ColumnVector& xdot,
       if (tlen == 0 || ! tmp(0).is_defined ())
         err_user_supplied_eval ("dassl");
 
-      if (! warned_fcn_imaginary && tmp(0).is_complex_type ())
+      if (! warned_fcn_imaginary && tmp(0).iscomplex ())
         {
           warning ("dassl: ignoring imaginary part returned from user-supplied function");
           warned_fcn_imaginary = true;
@@ -99,7 +100,7 @@ dassl_user_function (const ColumnVector& x, const ColumnVector& xdot,
       if (tlen > 1)
         ires = tmp(1).int_value ();
 
-      if (retval.is_empty ())
+      if (retval.isempty ())
         err_user_supplied_eval ("dassl");
     }
 
@@ -127,7 +128,7 @@ dassl_user_jacobian (const ColumnVector& x, const ColumnVector& xdot,
 
       try
         {
-          tmp = dassl_jac->do_multi_index_op (1, args);
+          tmp = octave::feval (dassl_jac, args, 1);
         }
       catch (octave::execution_exception& e)
         {
@@ -138,7 +139,7 @@ dassl_user_jacobian (const ColumnVector& x, const ColumnVector& xdot,
       if (tlen == 0 || ! tmp(0).is_defined ())
         err_user_supplied_eval ("dassl");
 
-      if (! warned_jac_imaginary && tmp(0).is_complex_type ())
+      if (! warned_jac_imaginary && tmp(0).iscomplex ())
         {
           warning ("dassl: ignoring imaginary part returned from user-supplied jacobian function");
           warned_jac_imaginary = true;
@@ -146,17 +147,19 @@ dassl_user_jacobian (const ColumnVector& x, const ColumnVector& xdot,
 
       retval = tmp(0).matrix_value ();
 
-      if (retval.is_empty ())
+      if (retval.isempty ())
         err_user_supplied_eval ("dassl");
     }
 
   return retval;
 }
 
-DEFUN (dassl, args, nargout,
-       doc: /* -*- texinfo -*-
+DEFMETHOD (dassl, interp, args, nargout,
+           doc: /* -*- texinfo -*-
 @deftypefn {} {[@var{x}, @var{xdot}, @var{istate}, @var{msg}] =} dassl (@var{fcn}, @var{x_0}, @var{xdot_0}, @var{t}, @var{t_crit})
-Solve the set of differential-algebraic equations
+Solve a set of differential-algebraic equations.
+
+@code{dassl} solves the set of equations
 @tex
 $$ 0 = f (x, \dot{x}, t) $$
 with
@@ -273,13 +276,15 @@ parameters for @code{dassl}.
   if (call_depth > 1)
     error ("dassl: invalid recursive call");
 
+  octave::symbol_table& symtab = interp.get_symbol_table ();
+
   std::string fcn_name, fname, jac_name, jname;
-  dassl_fcn = 0;
-  dassl_jac = 0;
+  dassl_fcn = nullptr;
+  dassl_jac = nullptr;
 
   octave_value f_arg = args(0);
 
-  if (f_arg.is_cell ())
+  if (f_arg.iscell ())
     {
       Cell c = f_arg.cell_value ();
       if (c.numel () == 1)
@@ -314,8 +319,8 @@ parameters for @code{dassl}.
                   if (! dassl_jac)
                     {
                       if (fcn_name.length ())
-                        clear_function (fcn_name);
-                      dassl_fcn = 0;
+                        symtab.clear_function (fcn_name);
+                      dassl_fcn = nullptr;
                     }
                 }
             }
@@ -324,7 +329,7 @@ parameters for @code{dassl}.
         error ("dassl: incorrect number of elements in cell array");
     }
 
-  if (! dassl_fcn && ! f_arg.is_cell ())
+  if (! dassl_fcn && ! f_arg.iscell ())
     {
       if (f_arg.is_function_handle () || f_arg.is_inline_function ())
         dassl_fcn = f_arg.function_value ();
@@ -369,8 +374,8 @@ parameters for @code{dassl}.
                     if (! dassl_jac)
                       {
                         if (fcn_name.length ())
-                          clear_function (fcn_name);
-                        dassl_fcn = 0;
+                          symtab.clear_function (fcn_name);
+                        dassl_fcn = nullptr;
                       }
                   }
               }
@@ -418,9 +423,9 @@ parameters for @code{dassl}.
     output = dae.integrate (out_times, deriv_output);
 
   if (fcn_name.length ())
-    clear_function (fcn_name);
+    symtab.clear_function (fcn_name);
   if (jac_name.length ())
-    clear_function (jac_name);
+    symtab.clear_function (jac_name);
 
   std::string msg = dae.error_message ();
 
@@ -529,4 +534,3 @@ parameters for @code{dassl}.
 
 %!error dassl_options ("foo", 1, 2)
 */
-
