@@ -1192,7 +1192,6 @@ write_header (std::ostream& os, load_save_format format)
     case LS_MAT7_BINARY:
       {
         char const *versionmagic;
-        int16_t number = *(reinterpret_cast<const int16_t *>("\x00\x01"));
         char headertext[128];
         octave::sys::gmtime now;
 
@@ -1212,7 +1211,7 @@ write_header (std::ostream& os, load_save_format format)
         // pair indicates whether the file was written by a big- or
         // little-endian machine.  However, the version number is
         // written in the *opposite* byte order from everything else!
-        if (number == 1)
+        if (octave::mach_info::words_big_endian ())
           versionmagic = "\x01\x00\x4d\x49"; // this machine is big endian
         else
           versionmagic = "\x00\x01\x49\x4d"; // this machine is little endian
@@ -1799,8 +1798,8 @@ Query or set the internal variable that specifies the maximum amount of memory
 that Octave will save when writing a crash dump file.
 
 The limit is measured in kilobytes and is applied to the top-level workspace.
-The name of the crash dump file is is specified by
-@var{octave_core_file_name}).
+The name of the crash dump file is specified by
+@var{octave_core_file_name}.
 
 If @var{octave_core_file_options} flags specify a binary format, then
 @var{octave_core_file_limit} will be approximately the maximum size of the
