@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 1993-2017 John W. Eaton
+Copyright (C) 1993-2018 John W. Eaton
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -24,11 +24,8 @@ along with Octave; see the file COPYING.  If not, see
 #  include "config.h"
 #endif
 
-#include <cfloat>
+#include <cmath>
 #include <cstddef>
-#include <cstdio>
-#include <cstdlib>
-#include <cstring>
 
 #include <iostream>
 #include <string>
@@ -64,17 +61,15 @@ along with Octave; see the file COPYING.  If not, see
 #include "cmd-edit.h"
 #include "file-ops.h"
 #include "lo-mappers.h"
-#include "lo-math.h"
+#include "lo-sysinfo.h"
 #include "mach-info.h"
 #include "oct-env.h"
-#include "quit.h"
 #include "unistd-wrappers.h"
 #include "unsetenv-wrapper.h"
 
 #include "builtin-defun-decls.h"
 #include "Cell.h"
 #include "defun.h"
-#include "display.h"
 #include "error.h"
 #include "errwarn.h"
 #include "input.h"
@@ -141,8 +136,8 @@ w32_set_octave_home (void)
               if (mod_name.find ("octinterp") != std::string::npos)
                 {
                   bin_dir = mod_info.szExePath;
-                  if (bin_dir[bin_dir.length () - 1] != '\\')
-                    bin_dir.append (1, '\\');
+                  if (! bin_dir.empty () && bin_dir.back () != '\\')
+                    bin_dir.push_back ('\\');
                   break;
                 }
             }
@@ -154,7 +149,7 @@ w32_set_octave_home (void)
 
   if (! bin_dir.empty ())
     {
-      size_t pos = bin_dir.rfind ("\\bin\\");
+      size_t pos = bin_dir.rfind (R"(\bin\)");
 
       if (pos != std::string::npos)
         octave::sys::env::putenv ("OCTAVE_HOME", bin_dir.substr (0, pos));
@@ -169,10 +164,6 @@ w32_init (void)
   octave::command_editor::prefer_env_winsize (true);
 }
 
-static bool
-w32_shell_execute (const std::string& file)
-{ }
-
 #endif
 
 // Set app id if we have the SetCurrentProcessExplicitAppUserModelID
@@ -189,7 +180,7 @@ set_application_id (void)
 
   HMODULE hShell = LoadLibrary ("shell32.dll");
 
-  if (hShell != NULL)
+  if (hShell)
     {
       SETCURRENTAPPID pfnSetCurrentProcessExplicitAppUserModelID =
         reinterpret_cast<SETCURRENTAPPID> (GetProcAddress (hShell,
@@ -410,7 +401,7 @@ raw_mode (bool on, bool wait)
 #if defined (ONLRET)
         s.c_oflag &= ~(ONLRET);
 #endif
-        s.c_cc[VMIN] = wait ? 1 : 0;
+        s.c_cc[VMIN] = (wait ? 1 : 0);
         s.c_cc[VTIME] = 0;
       }
     else
@@ -453,7 +444,7 @@ raw_mode (bool on, bool wait)
 #if defined (ONLRET)
         s.c_oflag &= ~(ONLRET);
 #endif
-        s.c_cc[VMIN] = wait ? 1 : 0;
+        s.c_cc[VMIN] = (wait ? 1 : 0);
       }
     else
       {
@@ -466,6 +457,8 @@ raw_mode (bool on, bool wait)
   }
 #elif defined (HAVE_SGTTY_H)
   {
+    octave_unused_parameter (wait);
+
     struct sgttyb s;
     static struct sgttyb save_term;
 
@@ -497,6 +490,9 @@ raw_mode (bool on, bool wait)
     ioctl (tty_fd, TIOCSETN, &s);
   }
 #else
+
+  octave_unused_parameter (wait);
+
   warn_disabled_feature ("", "raw mode console I/O");
 
   // Make sure the current mode doesn't toggle.
@@ -512,10 +508,8 @@ octave_popen (const char *command, const char *mode)
 #if defined (__MINGW32__) || defined (_MSC_VER)
   if (mode && mode[0] && ! mode[1])
     {
-      char tmode[3];
-      tmode[0] = mode[0];
-      tmode[1] = 'b';
-      tmode[2] = 0;
+      // Use binary mode on Windows if unspecified
+      char tmode[3] = {mode[0], 'b', '\0'};
 
       return _popen (command, tmode);
     }
@@ -591,7 +585,7 @@ get_P_tmpdir (void)
   // P_tmpdir, or they define it to a single backslash, neither of which
   // is particularly helpful.
 
-  if (retval.empty () || retval == "\\")
+  if (retval.empty () || retval == R"(\)")
     {
       retval = octave::sys::env::getenv ("TEMP");
 
@@ -599,7 +593,7 @@ get_P_tmpdir (void)
         retval = octave::sys::env::getenv ("TMP");
 
       if (retval.empty ())
-        retval = "c:\\temp";
+        retval = R"(c:\temp)";
     }
 
   return retval;
@@ -719,6 +713,289 @@ occurred.
 ## Test for unsetenv is in setenv test
 */
 
+#if defined (OCTAVE_USE_WINDOWS_API)
+
+static void
+reg_close_key_wrapper (HKEY key)
+{
+  RegCloseKey (key);
+}
+
+LONG
+get_regkey_value (HKEY h_rootkey, const std::string subkey,
+                  const std::string name, octave_value& value)
+{
+  LONG result;
+  HKEY h_subkey;
+
+  result = RegOpenKeyExA (h_rootkey, subkey.c_str (), 0, KEY_READ, &h_subkey);
+  if (result != ERROR_SUCCESS)
+    return result;
+
+  octave::unwind_protect frame;
+
+  frame.add_fcn (reg_close_key_wrapper, h_subkey);
+
+  DWORD length = 0;
+  result = RegQueryValueExA (h_subkey, name.c_str (), nullptr, nullptr, nullptr,
+                             &length);
+  if (result != ERROR_SUCCESS)
+    return result;
+
+  DWORD type = 0;
+  OCTAVE_LOCAL_BUFFER (BYTE, data, length);
+  result = RegQueryValueExA (h_subkey, name.c_str (), nullptr, &type, data,
+                             &length);
+  if (result != ERROR_SUCCESS)
+    return result;
+
+  if (type == REG_DWORD)
+    value = octave_int32 (*data);
+  else if (type == REG_SZ || type == REG_EXPAND_SZ)
+    value = string_vector (reinterpret_cast<char *> (data));
+
+  return result;
+}
+
+LONG
+get_regkey_names (HKEY h_rootkey, const std::string subkey,
+                  std::list<std::string> &fields)
+{
+  LONG retval;
+  HKEY h_subkey;
+
+  fields.clear ();
+
+  retval = RegOpenKeyEx (h_rootkey, subkey.c_str (), 0, KEY_READ, &h_subkey);
+  if (retval != ERROR_SUCCESS)
+    return retval;
+
+  DWORD idx = 0;
+  const int MAX_VALUE_NAME_SIZE = 32766;
+  char value_name[MAX_VALUE_NAME_SIZE+1];
+  DWORD value_name_size = MAX_VALUE_NAME_SIZE;
+
+  while (true)
+    {
+      retval = RegEnumValue (h_subkey, idx, value_name, &value_name_size,
+                             nullptr, nullptr, nullptr, nullptr);
+      if (retval != ERROR_SUCCESS)
+        break;
+      fields.push_back (value_name);
+      value_name_size = MAX_VALUE_NAME_SIZE;
+      idx++;
+    }
+
+  if (retval == ERROR_NO_MORE_ITEMS)
+    retval = ERROR_SUCCESS;
+
+  RegCloseKey (h_subkey);
+
+  return retval;
+}
+#endif
+
+DEFUN (winqueryreg, args, ,
+       doc: /* -*- texinfo -*-
+@deftypefn  {} {@var{value} =} winqueryreg (@var{rootkey}, @var{subkey}, @var{valuename})
+@deftypefnx {} {@var{value} =} winqueryreg (@var{rootkey}, @var{subkey})
+@deftypefnx {} {@var{names} =} winqueryreg (@code{"name"}, @var{rootkey}, @var{subkey})
+
+Query names or value from the Windows registry.
+
+On Windows, return the value of the registry key @var{subkey} from the root key
+@var{rootkey}.  You can specify the name of the queried registry value with the
+optional argument @var{valuename}.  Otherwise, if called with only two
+arguments or @var{valuename} is empty, then the default value of @var{subkey}
+is returned.  If the registry value is of type @nospell{@qcode{"REG_DWORD"}}
+then @var{value} is of class int32.  If the value is of the type
+@nospell{@qcode{"REG_SZ"}} or @nospell{@qcode{"REG_EXPAND_SZ"}} a string is
+returned.
+
+If the first argument is @qcode{"name"}, a cell array of strings with the names
+of the values at that key is returned.
+
+The variable @var{rootkey} must be a string with a valid root key identifier:
+
+@table @asis
+@item  @nospell{HKCR}
+@itemx @nospell{HKEY_CLASSES_ROOT}
+
+@item @nospell{HKEY_CURRENT_CONFIG}
+
+@item  @nospell{HKCU}
+@itemx @nospell{HKEY_CURRENT_USER}
+
+@item  @nospell{HKLM}
+@itemx @nospell{HKEY_LOCAL_MACHINE}
+
+
+@item  @nospell{HKU}
+@itemx @nospell{HKEY_USERS}
+
+
+@item @nospell{HKEY_PERFORMANCE_DATA}
+
+@end table
+
+Examples:
+
+Get a list of value names at the key @nospell{@qcode{'HKCU\Environment'}}:
+
+@example
+@group
+@var{valuenames} = winqueryreg ("name", "HKEY_CURRENT_USER", ...
+                          "Environment");
+@end group
+@end example
+
+For each @var{valuenames}, display the value:
+
+@example
+@group
+for @var{k} = 1:numel (@var{valuenames})
+  @var{val} = winqueryreg ("HKEY_CURRENT_USER", "Environment", ...
+                     @var{valuenames}@{@var{k}@});
+  @var{str} = sprintf ("%s = %s", @var{valuenames}@{@var{k}@}, num2str (@var{val}));
+  disp (@var{str});
+endfor
+@end group
+@end example
+
+On non-Windows platforms this function fails with an error.
+@end deftypefn */)
+{
+#if defined (OCTAVE_USE_WINDOWS_API)
+  if ((args.length () < 2) || (args.length () > 3))
+    print_usage ();
+
+  // Input check
+  std::string rootkey_name =
+    args(0).xstring_value ("winqueryreg: the first argument must be 'name' "
+                           "or a valid ROOTKEY identifier");
+  std::string subkey_name = "";
+  std::string value_name = "";
+  bool get_names = false;
+  if (rootkey_name.compare ("name") == 0)
+    {
+      if (args.length () < 3)
+        error ("winqueryreg: if the first argument is 'name', "
+               "ROOTKEY and SUBKEY must be given");
+      get_names = true;
+      rootkey_name =
+        args(1).xstring_value ("winqueryreg: ROOTKEY must be a string");
+      subkey_name =
+        args(2).xstring_value ("winqueryreg: SUBKEY must be a string");
+    }
+  else
+    {
+      subkey_name =
+        args(1).xstring_value ("winqueryreg: SUBKEY must be a string");
+      if (args.length () == 3)
+        value_name =
+          args(2).xstring_value ("winqueryreg: VALUENAME must be a string");
+    }
+
+  // Get rootkey handle
+  HKEY h_rootkey;
+  if (rootkey_name == "HKEY_CLASSES_ROOT" || rootkey_name == "HKCR")
+    h_rootkey = HKEY_CLASSES_ROOT;
+  else if (rootkey_name == "HKEY_CURRENT_CONFIG")
+    h_rootkey = HKEY_CURRENT_CONFIG;
+  else if (rootkey_name == "HKEY_CURRENT_USER" || rootkey_name == "HKCU")
+    h_rootkey = HKEY_CURRENT_USER;
+  else if (rootkey_name == "HKEY_LOCAL_MACHINE" || rootkey_name == "HKLM")
+    h_rootkey = HKEY_LOCAL_MACHINE;
+  else if (rootkey_name == "HKEY_PERFORMANCE_DATA")
+    h_rootkey = HKEY_PERFORMANCE_DATA;
+  else if (rootkey_name == "HKEY_USERS" || rootkey_name == "HKU")
+    h_rootkey = HKEY_USERS;
+  else
+    error ("winqueryreg: ROOTKEY is not a valid root key identifier");
+
+  if (get_names)
+    {
+      std::list<std::string> fields;
+
+      LONG retval = get_regkey_names (h_rootkey, subkey_name, fields);
+      if (retval != ERROR_SUCCESS)
+        error ("winqueryreg: error %d reading names from registry", retval);
+
+      Cell fieldnames (dim_vector (1, fields.size ()));
+      size_t i;
+      std::list<std::string>::const_iterator it;
+      for (i = 0, it = fields.begin (); it != fields.end (); ++it, ++i)
+        fieldnames(i) = *it;
+
+      return ovl (fieldnames);
+    }
+  else
+    {
+      octave_value key_val;
+      LONG retval = get_regkey_value (h_rootkey, subkey_name, value_name,
+                                      key_val);
+      if (retval == ERROR_FILE_NOT_FOUND)
+        error ("winqueryreg: no value found for '%s' at %s\\%s.",
+               value_name.c_str (), rootkey_name.c_str (),
+               subkey_name.c_str ());
+      if (retval != ERROR_SUCCESS)
+        error ("winqueryreg: error %d reading the specified key", retval);
+
+      return ovl (key_val);
+    }
+#else
+
+  octave_unused_parameter (args);
+
+  error ("winqueryreg: function is only supported on Windows platforms");
+
+#endif
+}
+
+/*
+%!testif ; ispc ()
+%! assert (ischar (winqueryreg ("HKEY_LOCAL_MACHINE",
+%!                              'SOFTWARE\Microsoft\Windows\CurrentVersion',
+%!                              "ProgramFilesDir")));
+%!testif ; ispc ()
+%! assert (isa (winqueryreg ("HKEY_LOCAL_MACHINE",
+%!                           'SYSTEM\CurrentControlSet\Control\FileSystem',
+%!                           "Win31FileSystem"), "int32"));
+%!testif ; ispc ()
+%! assert (iscellstr (winqueryreg ("name", "HKEY_LOCAL_MACHINE",
+%!                                 'SOFTWARE\Microsoft\Windows\CurrentVersion')));
+%!testif ; ispc ()
+%! fail ('winqueryreg (1, ''SOFTWARE\Microsoft\Windows\CurrentVersion'')',
+%!       "first argument must be 'name' or a valid ROOTKEY identifier");
+%!testif ; ispc ()
+%! fail ('winqueryreg ("HKEY_LOCAL_MACHINE", 1)', "SUBKEY must be a string");
+%!testif ; ispc ()
+%! fail (['winqueryreg ("HKEY_LOCAL_MACHINE", ', ...
+%!        '''SOFTWARE\Microsoft\Windows\CurrentVersion'', 1)'],
+%!       "VALUENAME must be a string");
+%!testif ; ispc ()
+%! fail (['winqueryreg ("FOO", ', ...
+%!        '''SOFTWARE\Microsoft\Windows\CurrentVersion'')'],
+%!       "ROOTKEY is not a valid root key identifier");
+%!testif ; ispc ()
+%! fail ('winqueryreg ("HKEY_LOCAL_MACHINE", ''FOO\bar'')',
+%!       "no value found for");
+%!testif ; ispc ()
+%! fail (['winqueryreg ("HKEY_LOCAL_MACHINE", ', ...
+%!        '''SOFTWARE\Microsoft\Windows\CurrentVersion'', "foo")'],
+%!       "no value found for");
+%!testif ; ispc ()
+%! fail ('winqueryreg ("name", "HKEY_LOCAL_MACHINE")',
+%!       "if the first argument is 'name', ROOTKEY and SUBKEY must be given");
+%!testif ; ispc ()
+%! fail (['winqueryreg ("name", 1, ', ...
+%!        '''SOFTWARE\Microsoft\Windows\CurrentVersion'')'],
+%!       "ROOTKEY must be a string");
+%!testif ; ispc ()
+%! fail ('winqueryreg ("name", "HKEY_LOCAL_MACHINE", 1)',
+%!       "SUBKEY must be a string");
+*/
+
 // FIXME: perhaps kbhit should also be able to print a prompt?
 
 DEFUN (kbhit, args, ,
@@ -749,25 +1026,18 @@ returning the empty string if no key is available.
 @seealso{input, pause}
 @end deftypefn */)
 {
-  octave_value retval = "";
-
   // FIXME: add timeout and default value args?
 
-  if (octave::application::interactive ())
-    {
-      Fdrawnow ();
+  Fdrawnow ();
 
-      int c = octave_kbhit (args.length () == 0);
+  int c = octave_kbhit (args.length () == 0);
 
-      if (c == -1)
-        c = 0;
+  if (c == -1)
+    c = 0;
 
-      char s[2] = { static_cast<char> (c), '\0' };
+  char s[2] = { static_cast<char> (c), '\0' };
 
-      retval = s;
-    }
-
-  return retval;
+  return octave_value (s);
 }
 
 DEFUN (pause, args, ,
@@ -820,7 +1090,7 @@ clc;
 
           if (octave::math::isinf (dval))
             {
-              flush_octave_stdout ();
+              octave::flush_stdout ();
               octave_kbhit ();
             }
           else
@@ -830,7 +1100,7 @@ clc;
   else
     {
       Fdrawnow ();
-      flush_octave_stdout ();
+      octave::flush_stdout ();
       octave_kbhit ();
     }
 
@@ -920,7 +1190,7 @@ tilde_expand ("~/bin")
 
   sv = octave::sys::file_ops::tilde_expand (sv);
 
-  if (arg.is_cellstr ())
+  if (arg.iscellstr ())
     return ovl (Cell (arg.dims (), sv));
   else
     return ovl (sv);
@@ -956,17 +1226,20 @@ equivalent to
 %! endif
 */
 
-// This function really belongs in display.cc, but including defun.h in
-// that file results in conflicts with symbols from headers that are
-// needed for X11 and Carbon functions.
-
-DEFUN (have_window_system, , ,
+DEFUN (__blas_version__, , ,
        doc: /* -*- texinfo -*-
-@deftypefn {} {} have_window_system ()
-Return true if a window system is available (X11, Windows, or Apple OS X)
-and false otherwise.
-@seealso{isguirunning}
+@deftypefn {} {} __blas_version__ ()
+Undocumented internal function.
 @end deftypefn */)
 {
-  return ovl (display_info::display_available ());
+  return ovl (octave::sys::blas_version ());
+}
+
+DEFUN (__lapack_version__, , ,
+       doc: /* -*- texinfo -*-
+@deftypefn {} {} __lapack_version__ ()
+Undocumented internal function.
+@end deftypefn */)
+{
+  return ovl (octave::sys::lapack_version ());
 }
