@@ -1,23 +1,23 @@
 /*
 
-Copyright (C) 1999-2017 John W. Eaton
+Copyright (C) 1999-2018 John W. Eaton
 Copyright (C) 2009-2010 VZLU Prague
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -42,8 +42,6 @@ along with Octave; see the file COPYING.  If not, see
 
 class octave_value_list;
 
-class tree_walker;
-
 // Cells.
 
 class
@@ -64,13 +62,13 @@ public:
   octave_cell (const octave_cell& c)
     : octave_base_matrix<Cell> (c), cellstr_cache () { }
 
-  ~octave_cell (void) { }
+  ~octave_cell (void) = default;
 
-  octave_base_value *clone (void) const { return new octave_cell (*this); }
-  octave_base_value *empty_clone (void) const { return new octave_cell (); }
+  octave_base_value * clone (void) const { return new octave_cell (*this); }
+  octave_base_value * empty_clone (void) const { return new octave_cell (); }
 
 #if 0
-  octave_base_value *try_narrowing_conversion (void);
+  octave_base_value * try_narrowing_conversion (void);
 #endif
 
   octave_value subsref (const std::string& type,
@@ -82,15 +80,7 @@ public:
 
   octave_value_list subsref (const std::string& type,
                              const std::list<octave_value_list>& idx,
-                             int nargout)
-  {
-    return subsref (type, idx, nargout, 0);
-  }
-
-  octave_value_list subsref (const std::string& type,
-                             const std::list<octave_value_list>& idx,
-                             int nargout,
-                             const std::list<octave_lvalue> *lvalue_list);
+                             int nargout);
 
   octave_value subsref (const std::string& type,
                         const std::list<octave_value_list>& idx,
@@ -113,7 +103,7 @@ public:
   octave_value sort (Array<octave_idx_type> &sidx, octave_idx_type dim = 0,
                      sortmode mode = ASCENDING) const;
 
-  sortmode is_sorted (sortmode mode = UNSORTED) const;
+  sortmode issorted (sortmode mode = UNSORTED) const;
 
   Array<octave_idx_type> sort_rows_idx (sortmode mode = ASCENDING) const;
 
@@ -121,17 +111,17 @@ public:
 
   bool is_matrix_type (void) const { return false; }
 
-  bool is_numeric_type (void) const { return false; }
+  bool isnumeric (void) const { return false; }
 
   bool is_defined (void) const { return true; }
 
   bool is_constant (void) const { return true; }
 
-  bool is_cell (void) const { return true; }
+  bool iscell (void) const { return true; }
 
   builtin_type_t builtin_type (void) const { return btyp_cell; }
 
-  bool is_cellstr (void) const;
+  bool iscellstr (void) const;
 
   bool is_true (void) const;
 
@@ -154,6 +144,8 @@ public:
 
   void print_raw (std::ostream& os, bool pr_as_read_syntax = false) const;
 
+  bool print_name_tag (std::ostream& os, const std::string& name) const;
+
   void short_disp (std::ostream& os) const;
 
   bool save_ascii (std::ostream& os);
@@ -171,18 +163,18 @@ public:
 
   octave_value map (unary_mapper_t umap) const;
 
-  mxArray *as_mxArray (void) const;
+  mxArray * as_mxArray (void) const;
 
   // Unsafe.  This function exists to support the MEX interface.
   // You should not use it anywhere else.
-  void *mex_get_data (void) const;
+  void * mex_get_data (void) const;
 
 private:
 
   void clear_cellstr_cache (void) const
   { cellstr_cache.reset (); }
 
-  mutable std::unique_ptr<Array<std::string> > cellstr_cache;
+  mutable std::unique_ptr<Array<std::string>> cellstr_cache;
 
   DECLARE_OV_TYPEID_FUNCTIONS_AND_DATA
 };
