@@ -1,36 +1,39 @@
 /*
 
-Copyright (C) 2012-2015 Michael Goffioul
+Copyright (C) 2012-2018 Michael Goffioul
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
-#if !defined (octave_classdef_h)
+#if ! defined (octave_classdef_h)
 #define octave_classdef_h 1
 
+#include "octave-config.h"
+
 #include <map>
 #include <set>
 #include <string>
 
-#include "oct-map.h"
 #include "oct-refcount.h"
+
+#include "oct-map.h"
 #include "ov-base.h"
-#include "symtab.h"
+#include "ov-builtin.h"
 
 class cdef_object;
 class cdef_class;
@@ -38,7 +41,12 @@ class cdef_property;
 class cdef_method;
 class cdef_package;
 
-class tree_classdef;
+namespace octave
+{
+  class interpreter;
+  class tree_classdef;
+  class type_info;
+}
 
 // This is mainly a boostrap class to declare the expected interface.
 // The actual base class is cdef_class_base, which is declared after
@@ -50,37 +58,37 @@ public:
   friend class cdef_object;
 
 public:
-  cdef_object_rep (void) : refcount (1) { }
 
-  virtual ~cdef_object_rep (void) { }
+  cdef_object_rep (void) : refcount (1)
+  { }
+
+  cdef_object_rep& operator = (const cdef_object_rep&) = delete;
+
+  virtual ~cdef_object_rep (void) = default;
 
   virtual cdef_class get_class (void) const;
 
   virtual void set_class (const cdef_class&)
-  { gripe_invalid_object ("set_class"); }
+  { err_invalid_object ("set_class"); }
 
-  virtual cdef_object_rep* clone (void) const
+  virtual cdef_object_rep * clone (void) const
   {
-    gripe_invalid_object ("clone");
-    return new cdef_object_rep ();
+    err_invalid_object ("clone");
   }
 
-  virtual cdef_object_rep* empty_clone (void) const
+  virtual cdef_object_rep * empty_clone (void) const
   {
-    gripe_invalid_object ("empty_clone");
-    return new cdef_object_rep ();
+    err_invalid_object ("empty_clone");
   }
 
-  virtual cdef_object_rep* copy (void) const
+  virtual cdef_object_rep * copy (void) const
   {
-    gripe_invalid_object ("copy");
-    return new cdef_object_rep ();
+    err_invalid_object ("copy");
   }
 
-  virtual cdef_object_rep* make_array (void) const
+  virtual cdef_object_rep * make_array (void) const
   {
-    gripe_invalid_object ("make_array");
-    return new cdef_object_rep ();
+    err_invalid_object ("make_array");
   }
 
   virtual bool is_array (void) const { return false; }
@@ -93,33 +101,40 @@ public:
 
   virtual Array<cdef_object> array_value (void) const
   {
-    gripe_invalid_object ("array_value");
-    return Array<cdef_object> ();
+    err_invalid_object ("array_value");
   }
 
   virtual void put (const std::string&, const octave_value&)
-  { gripe_invalid_object ("put"); }
+  { err_invalid_object ("put"); }
 
   virtual octave_value get (const std::string&) const
   {
-    gripe_invalid_object ("get");
-    return octave_value ();
+    err_invalid_object ("get");
+  }
+
+  virtual void set_property (octave_idx_type, const std::string&,
+                             const octave_value&)
+  {
+    err_invalid_object ("set_property");
+  }
+
+  virtual octave_value get_property (octave_idx_type, const std::string&) const
+  {
+    err_invalid_object ("get_property");
   }
 
   virtual octave_value_list
   subsref (const std::string&, const std::list<octave_value_list>&,
            int, size_t&, const cdef_class&, bool)
   {
-    gripe_invalid_object ("subsref");
-    return octave_value_list ();
+    err_invalid_object ("subsref");
   }
 
   virtual octave_value
   subsasgn (const std::string&, const std::list<octave_value_list>&,
             const octave_value&)
   {
-    gripe_invalid_object ("subsasgn");
-    return octave_value ();
+    err_invalid_object ("subsasgn");
   }
 
   virtual string_vector map_keys (void) const;
@@ -129,58 +144,59 @@ public:
   std::string class_name (void) const;
 
   virtual void mark_for_construction (const cdef_class&)
-  { gripe_invalid_object ("mark_for_construction"); }
+  {
+    err_invalid_object ("mark_for_construction");
+  }
 
   virtual bool is_constructed_for (const cdef_class&) const
   {
-    gripe_invalid_object ("is_constructed_for");
-    return false;
+    err_invalid_object ("is_constructed_for");
   }
 
   virtual bool is_partially_constructed_for (const cdef_class&) const
   {
-    gripe_invalid_object ("is_partially_constructed_for");
-    return false;
+    err_invalid_object ("is_partially_constructed_for");
   }
 
   virtual void mark_as_constructed (void)
-  { gripe_invalid_object ("mark_as_constructed"); }
+  {
+    err_invalid_object ("mark_as_constructed");
+  }
 
   virtual void mark_as_constructed (const cdef_class&)
-  { gripe_invalid_object ("mark_as_constructed"); }
+  {
+    err_invalid_object ("mark_as_constructed");
+  }
 
   virtual bool is_constructed (void) const
   {
-    gripe_invalid_object ("is_constructed");
-    return false;
+    err_invalid_object ("is_constructed");
   }
 
   virtual octave_idx_type static_count (void) const { return 0; }
 
   virtual void destroy (void) { delete this; }
 
-  void release (void)
-  {
-    if (--refcount == static_count ())
-      destroy ();
-  }
+  void release (const cdef_object& obj);
 
   virtual dim_vector dims (void) const { return dim_vector (); }
 
 protected:
-  /* reference count */
-  octave_refcount<octave_idx_type> refcount;
+
+  // Reference count
+  octave::refcount<octave_idx_type> refcount;
 
 protected:
-  /* Restricted copying */
+
+  // Restricted copying.
+
   cdef_object_rep (const cdef_object_rep&)
-    : refcount (1) { }
+    : refcount (1)
+  { }
 
 private:
-  /* No assignment */
-  cdef_object_rep& operator = (const cdef_object_rep& );
 
-  void gripe_invalid_object (const char *who) const
+  OCTAVE_NORETURN void err_invalid_object (const char *who) const
   { error ("%s: invalid object", who); }
 };
 
@@ -188,27 +204,26 @@ class
 cdef_object
 {
 public:
-  /* FIXME: use a null object */
+  // FIXME: use a null object
   cdef_object (void)
     : rep (new cdef_object_rep ()) { }
 
   cdef_object (const cdef_object& obj)
-    : rep (obj.rep)
-  {
-    rep->refcount++;
-  }
+    : rep (obj.rep) { rep->refcount++; }
 
   cdef_object (cdef_object_rep *r)
     : rep (r) { }
 
   virtual ~cdef_object (void)
-  { rep->release (); }
+  {
+    rep->release (*this);
+  }
 
   cdef_object& operator = (const cdef_object& obj)
   {
     if (rep != obj.rep)
       {
-        rep->release ();
+        rep->release (*this);
 
         rep = obj.rep;
         rep->refcount++;
@@ -254,6 +269,14 @@ public:
   octave_value get (const std::string& pname) const
   { return rep->get (pname); }
 
+  void set_property (octave_idx_type idx, const std::string& pname,
+                     const octave_value& pval)
+  { return rep->set_property (idx, pname, pval); }
+
+  octave_value
+  get_property (octave_idx_type idx, const std::string& pname) const
+  { return rep->get_property (idx, pname); }
+
   octave_value_list
   subsref (const std::string& type, const std::list<octave_value_list>& idx,
            int nargout, size_t& skip, const cdef_class& context,
@@ -272,7 +295,7 @@ public:
 
   octave_map map_value (void) const;
 
-  const cdef_object_rep* get_rep (void) const { return rep; }
+  const cdef_object_rep * get_rep (void) const { return rep; }
 
   bool ok (void) const { return rep->is_valid (); }
 
@@ -295,7 +318,7 @@ public:
   bool is (const cdef_object& obj) const { return rep == obj.rep; }
 
 protected:
-  cdef_object_rep* get_rep (void) { return rep; }
+  cdef_object_rep * get_rep (void) { return rep; }
 
   void make_unique (int ignore_copies)
   {
@@ -323,10 +346,10 @@ public:
 
   void set_class (const cdef_class& cls);
 
-  cdef_object_rep* empty_clone (void) const
+  cdef_object_rep * empty_clone (void) const
   { return new cdef_object_base (*this); }
 
-  cdef_object_rep* make_array (void) const;
+  cdef_object_rep * make_array (void) const;
 
 protected:
   // Restricted copying!
@@ -341,11 +364,9 @@ private:
 
   void unregister_object (void);
 
-private:
   // The class of the object
   cdef_object klass;
 
-private:
   // No assignment!
   cdef_object_base& operator = (const cdef_object_base&);
 };
@@ -359,7 +380,7 @@ public:
   cdef_object_array (const Array<cdef_object>& a)
     : cdef_object_base (), array (a) { }
 
-  cdef_object_rep* clone (void) const
+  cdef_object_rep * clone (void) const
   { return new cdef_object_array (*this); }
 
   dim_vector dims (void) const { return array.dims (); }
@@ -379,10 +400,25 @@ public:
   subsasgn (const std::string& type, const std::list<octave_value_list>& idx,
             const octave_value& rhs);
 
+  void set_property (octave_idx_type idx, const std::string& pname,
+                     const octave_value& pval)
+  {
+    cdef_object& tmp = array.elem (idx);
+
+    return tmp.put (pname, pval);
+  }
+
+  octave_value
+  get_property (octave_idx_type idx, const std::string& pname) const
+  {
+    cdef_object tmp = array.elem (idx);
+
+    return tmp.get (pname);
+  }
+
 private:
   Array<cdef_object> array;
 
-private:
   void fill_empty_values (void) { fill_empty_values (array); }
 
   void fill_empty_values (Array<cdef_object>& arr);
@@ -401,7 +437,7 @@ cdef_object_scalar : public cdef_object_base
 public:
   cdef_object_scalar (void) : cdef_object_base () { }
 
-  ~cdef_object_scalar (void) { }
+  ~cdef_object_scalar (void) = default;
 
   dim_vector dims (void) const { return dim_vector (1, 1); }
 
@@ -412,13 +448,28 @@ public:
   {
     Cell val = map.contents (pname);
 
-    if (val.numel () > 0)
-      return val(0, 0);
-    else
-      {
-        error ("get: unknown slot: %s", pname.c_str ());
-        return octave_value ();
-      }
+    if (val.numel () < 1)
+      error ("get: unknown slot: %s", pname.c_str ());
+
+    return val(0, 0);
+  }
+
+  void set_property (octave_idx_type idx, const std::string& pname,
+                     const octave_value& pval)
+  {
+    if (idx != 0)
+      error ("invalid index");  // FIXME
+
+    put (pname, pval);
+  }
+
+  octave_value
+  get_property (octave_idx_type idx, const std::string& pname) const
+  {
+    if (idx != 0)
+      error ("invalid index");  // FIXME
+
+    return get (pname);
   }
 
   octave_value_list
@@ -447,7 +498,7 @@ protected:
   octave_scalar_map map;
 
   // Internal/temporary structure used during object construction
-  std::map< cdef_class, std::list<cdef_class> > ctor_list;
+  std::map< cdef_class, std::list<cdef_class>> ctor_list;
 
 protected:
   // Restricted object copying!
@@ -468,14 +519,14 @@ public:
 
   ~handle_cdef_object (void);
 
-  cdef_object_rep* clone (void) const
+  cdef_object_rep * clone (void) const
   {
     handle_cdef_object *obj = const_cast<handle_cdef_object *> (this);
     obj->refcount++;
     return obj;
   }
 
-  cdef_object_rep* copy (void) const
+  cdef_object_rep * copy (void) const
   { return new handle_cdef_object (*this); }
 
   bool is_valid (void) const { return true; }
@@ -501,10 +552,10 @@ public:
 
   ~value_cdef_object (void);
 
-  cdef_object_rep* clone (void) const
+  cdef_object_rep * clone (void) const
   { return new value_cdef_object (*this); }
 
-  cdef_object_rep* copy (void) const { return clone (); }
+  cdef_object_rep * copy (void) const { return clone (); }
 
   bool is_valid (void) const { return true; }
 
@@ -526,9 +577,9 @@ public:
   cdef_meta_object_rep (void)
     : handle_cdef_object () { }
 
-  ~cdef_meta_object_rep (void) { }
+  ~cdef_meta_object_rep (void) = default;
 
-  cdef_object_rep* copy (void) const
+  cdef_object_rep * copy (void) const
   { return new cdef_meta_object_rep (*this); }
 
   bool is_meta_object (void) const { return true; }
@@ -546,13 +597,12 @@ public:
                 const std::list<octave_value_list>& /* idx */,
                 int /* nargout */)
   {
-    ::error ("subsref: invalid meta object");
-    return octave_value_list ();
+    error ("subsref: invalid meta object");
   }
 
   virtual void meta_release (void) { }
 
-  virtual bool meta_is_postfix_index_handled (char /* type */) const
+  virtual bool meta_accepts_postfix_index (char /* type */) const
   { return false; }
 
 protected:
@@ -572,17 +622,17 @@ public:
   cdef_meta_object (void)
     : cdef_object () { }
 
+  // Object consistency is checked in sub-classes.
   cdef_meta_object (const cdef_meta_object& obj)
     : cdef_object (obj) { }
 
   cdef_meta_object (cdef_meta_object_rep *r)
     : cdef_object (r) { }
 
-  // Object consistency is checked in sub-classes.
   cdef_meta_object (const cdef_object& obj)
     : cdef_object (obj) { }
 
-  ~cdef_meta_object (void) { }
+  ~cdef_meta_object (void) = default;
 
   bool is_class (void) const { return get_rep ()->is_class (); }
 
@@ -599,14 +649,14 @@ public:
 
   void meta_release (void) { get_rep ()->meta_release (); }
 
-  bool meta_is_postfix_index_handled (char type) const
-  { return get_rep ()->meta_is_postfix_index_handled (type); }
+  bool meta_accepts_postfix_index (char type) const
+  { return get_rep ()->meta_accepts_postfix_index (type); }
 
 private:
-  cdef_meta_object_rep* get_rep (void)
+  cdef_meta_object_rep * get_rep (void)
   { return dynamic_cast<cdef_meta_object_rep *> (cdef_object::get_rep ()); }
 
-  const cdef_meta_object_rep* get_rep (void) const
+  const cdef_meta_object_rep * get_rep (void) const
   { return dynamic_cast<const cdef_meta_object_rep *> (cdef_object::get_rep ()); }
 };
 
@@ -621,11 +671,12 @@ private:
   public:
     cdef_class_rep (void)
       : cdef_meta_object_rep (), member_count (0), handle_class (false),
-        object_count (0), meta (false) { }
+        object_count (0), meta (false)
+    { }
 
     cdef_class_rep (const std::list<cdef_class>& superclasses);
 
-    cdef_object_rep* copy (void) const { return new cdef_class_rep (*this); }
+    cdef_object_rep * copy (void) const { return new cdef_class_rep (*this); }
 
     bool is_class (void) const { return true; }
 
@@ -658,7 +709,7 @@ private:
 
     std::string get_directory (void) const { return directory; }
 
-    void delete_object (cdef_object obj);
+    void delete_object (const cdef_object& obj);
 
     octave_value_list
     meta_subsref (const std::string& type,
@@ -666,7 +717,7 @@ private:
 
     void meta_release (void);
 
-    bool meta_is_postfix_index_handled (char type) const
+    bool meta_accepts_postfix_index (char type) const
     { return (type == '(' || type == '.'); }
 
     octave_value construct (const octave_value_list& args);
@@ -707,6 +758,7 @@ private:
     bool is_meta_class (void) const { return meta; }
 
   private:
+
     void load_all_methods (void);
 
     void find_names (std::set<std::string>& names, bool all);
@@ -723,7 +775,6 @@ private:
       return cdef_class (this);
     }
 
-  private:
     // The @-directory were this class is loaded from.
     // (not used yet)
     std::string directory;
@@ -737,17 +788,17 @@ private:
     // The number of members in this class (methods, properties...)
     octave_idx_type member_count;
 
-    // TRUE if this class is a handle class. A class is a handle
+    // TRUE if this class is a handle class.  A class is a handle
     // class when the abstract "handle" class is one of its superclasses.
     bool handle_class;
 
     // The list of super-class constructors that are called implicitly by the
-    // the classdef engine when creating an object. These constructors are not
+    // the classdef engine when creating an object.  These constructors are not
     // called explicitly by the class constructor.
     std::list<cdef_class> implicit_ctor_list;
 
     // The number of objects of this class.
-    octave_refcount<octave_idx_type> object_count;
+    octave::refcount<octave_idx_type> object_count;
 
     // TRUE if this class is a built-in meta class.
     bool meta;
@@ -758,7 +809,6 @@ private:
     typedef std::map<std::string,cdef_property>::iterator property_iterator;
     typedef std::map<std::string,cdef_property>::const_iterator property_const_iterator;
 
-  private:
     cdef_class_rep (const cdef_class_rep& c)
       : cdef_meta_object_rep (c), directory (c.directory),
         method_map (c.method_map), property_map (c.property_map),
@@ -772,8 +822,7 @@ public:
   cdef_class (void)
     : cdef_meta_object () { }
 
-  cdef_class (const std::string& nm,
-              const std::list<cdef_class>& superclasses)
+  cdef_class (const std::string& nm, const std::list<cdef_class>& superclasses)
     : cdef_meta_object (new cdef_class_rep (superclasses))
   { get_rep ()->set_name (nm); }
 
@@ -791,7 +840,7 @@ public:
 
   cdef_class& operator = (const cdef_class& cls)
   {
-    cdef_object::operator= (cls);
+    cdef_object::operator = (cls);
 
     return *this;
   }
@@ -833,15 +882,36 @@ public:
   bool is_builtin (void) const
   { return get_directory ().empty (); }
 
-  void delete_object (cdef_object obj)
+  void delete_object (const cdef_object& obj)
   { get_rep ()->delete_object (obj); }
 
-  static cdef_class make_meta_class (tree_classdef* t,
-                                     bool is_at_folder = false);
+  //! Analyze the tree_classdef tree and transform it to a cdef_class
+  //!
+  //! <b>All attribute validation should occur here.</b>
+  //!
+  //! Classdef attribute values can be given in the form of
+  //! expressions.  These expressions must be evaluated before
+  //! assigning them as attribute values.  Evaluating them as they are
+  //! parsed causes trouble with possible recusion in the parser so we
+  //! do it here.  For example
+  //!
+  //! @code
+  //! classdef recursion_class
+  //!   methods (Access = ?recursion_class)
+  //!   endmethods
+  //! endclassdef
+  //! @endcode
+  //!
+  //! will fail because each attempt to compute the metaclass of
+  //! recursion_class will cause recursion_class to be parsed again.
 
-  octave_function* get_method_function (const std::string& nm);
+  static cdef_class
+  make_meta_class (octave::interpreter& interp, octave::tree_classdef *t,
+                   bool is_at_folder = false);
 
-  octave_function* get_constructor_function (void)
+  octave_function * get_method_function (const std::string& nm);
+
+  octave_function * get_constructor_function (void)
   { return get_method_function (get_name ()); }
 
   octave_value construct (const octave_value_list& args)
@@ -866,11 +936,6 @@ public:
 
   bool is_meta_class (void) const { return get_rep ()->is_meta_class (); }
 
-  static const cdef_class& meta_class (void) { return _meta_class; }
-  static const cdef_class& meta_property (void) { return _meta_property; }
-  static const cdef_class& meta_method (void) { return _meta_method; }
-  static const cdef_class& meta_package (void) { return _meta_package; }
-
   void register_object (void) { get_rep ()->register_object (); }
 
   void unregister_object (void) { get_rep ()->unregister_object (); }
@@ -884,23 +949,17 @@ public:
   };
 
 private:
-  cdef_class_rep* get_rep (void)
+  cdef_class_rep * get_rep (void)
   { return dynamic_cast<cdef_class_rep *> (cdef_object::get_rep ()); }
 
-  const cdef_class_rep* get_rep (void) const
+  const cdef_class_rep * get_rep (void) const
   { return dynamic_cast<const cdef_class_rep *> (cdef_object::get_rep ()); }
 
   friend bool operator == (const cdef_class&, const cdef_class&);
   friend bool operator != (const cdef_class&, const cdef_class&);
   friend bool operator < (const cdef_class&, const cdef_class&);
 
-private:
-  static cdef_class _meta_class;
-  static cdef_class _meta_property;
-  static cdef_class _meta_method;
-  static cdef_class _meta_package;
-
-  friend void install_classdef (void);
+  friend void install_classdef (octave::interpreter& interp);
 };
 
 inline bool
@@ -931,7 +990,7 @@ private:
     cdef_property_rep (void)
       : cdef_meta_object_rep () { }
 
-    cdef_object_rep* copy (void) const { return new cdef_property_rep (*this); }
+    cdef_object_rep * copy (void) const { return new cdef_property_rep (*this); }
 
     bool is_property (void) const { return true; }
 
@@ -942,15 +1001,15 @@ private:
     bool is_constant (void) const { return get("Constant").bool_value (); }
 
     octave_value get_value (bool do_check_access = true,
-                            const std::string& who = std::string ());
+                            const std::string& who = "");
 
     octave_value get_value (const cdef_object& obj,
                             bool do_check_access = true,
-                            const std::string& who = std::string ());
+                            const std::string& who = "");
 
     void set_value (cdef_object& obj, const octave_value& val,
                     bool do_check_access = true,
-                    const std::string& who = std::string ());
+                    const std::string& who = "");
 
     bool check_get_access (void) const;
 
@@ -990,22 +1049,22 @@ public:
 
   cdef_property& operator = (const cdef_property& prop)
   {
-    cdef_object::operator= (prop);
+    cdef_object::operator = (prop);
 
     return *this;
   }
 
   octave_value get_value (const cdef_object& obj, bool do_check_access = true,
-                          const std::string& who = std::string ())
+                          const std::string& who = "")
   { return get_rep ()->get_value (obj, do_check_access, who); }
 
   octave_value get_value (bool do_check_access = true,
-                          const std::string& who = std::string ())
+                          const std::string& who = "")
   { return get_rep ()->get_value (do_check_access, who); }
 
   void set_value (cdef_object& obj, const octave_value& val,
                   bool do_check_access = true,
-                  const std::string& who = std::string ())
+                  const std::string& who = "")
   { get_rep ()->set_value (obj, val, do_check_access, who); }
 
   bool check_get_access (void) const
@@ -1019,10 +1078,10 @@ public:
   bool is_constant (void) const { return get_rep ()->is_constant (); }
 
 private:
-  cdef_property_rep* get_rep (void)
+  cdef_property_rep * get_rep (void)
   { return dynamic_cast<cdef_property_rep *> (cdef_object::get_rep ()); }
 
-  const cdef_property_rep* get_rep (void) const
+  const cdef_property_rep * get_rep (void) const
   { return dynamic_cast<const cdef_property_rep *> (cdef_object::get_rep ()); }
 };
 
@@ -1041,7 +1100,7 @@ private:
       : cdef_meta_object_rep (), function (), dispatch_type ()
     { }
 
-    cdef_object_rep* copy (void) const { return new cdef_method_rep(*this); }
+    cdef_object_rep * copy (void) const { return new cdef_method_rep(*this); }
 
     bool is_method (void) const { return true; }
 
@@ -1064,12 +1123,12 @@ private:
 
     octave_value_list execute (const octave_value_list& args, int nargout,
                                bool do_check_access = true,
-                               const std::string& who = std::string ());
+                               const std::string& who = "");
 
     octave_value_list execute (const cdef_object& obj,
                                const octave_value_list& args, int nargout,
                                bool do_check_access = true,
-                               const std::string& who = std::string ());
+                               const std::string& who = "");
 
     bool is_constructor (void) const;
 
@@ -1077,7 +1136,7 @@ private:
     meta_subsref (const std::string& type,
                   const std::list<octave_value_list>& idx, int nargout);
 
-    bool meta_is_postfix_index_handled (char type) const
+    bool meta_accepts_postfix_index (char type) const
     { return (type == '(' || type == '.'); }
 
   private:
@@ -1094,7 +1153,6 @@ private:
       return cdef_method (this);
     }
 
-  private:
     octave_value function;
 
     // When non-empty, the method is externally defined and this member
@@ -1123,22 +1181,22 @@ public:
 
   cdef_method& operator = (const cdef_method& meth)
   {
-    cdef_object::operator= (meth);
+    cdef_object::operator = (meth);
 
     return *this;
   }
 
-  /* normal invokation */
+  // normal invocation
   octave_value_list execute (const octave_value_list& args, int nargout,
                              bool do_check_access = true,
-                             const std::string& who = std::string ())
+                             const std::string& who = "")
   { return get_rep ()->execute (args, nargout, do_check_access, who); }
 
-  /* dot-invokation: object is pushed as 1st argument */
+  // dot-invocation: object is pushed as 1st argument
   octave_value_list execute (const cdef_object& obj,
                              const octave_value_list& args, int nargout,
                              bool do_check_access = true,
-                             const std::string& who = std::string ())
+                             const std::string& who = "")
   { return get_rep ()->execute (obj, args, nargout, do_check_access, who); }
 
   bool check_access (void) const { return get_rep ()->check_access (); }
@@ -1162,18 +1220,17 @@ public:
   { get_rep ()->mark_as_external (dtype); }
 
 private:
-  cdef_method_rep* get_rep (void)
+  cdef_method_rep * get_rep (void)
   { return dynamic_cast<cdef_method_rep *> (cdef_object::get_rep ()); }
 
-  const cdef_method_rep* get_rep (void) const
+  const cdef_method_rep * get_rep (void) const
   { return dynamic_cast<const cdef_method_rep *> (cdef_object::get_rep ()); }
 };
 
 inline cdef_class
 cdef_object_rep::get_class (void) const
 {
-  gripe_invalid_object ("get_class");
-  return cdef_class ();
+  err_invalid_object ("get_class");
 }
 
 inline std::string
@@ -1208,7 +1265,7 @@ cdef_object_base::register_object (void)
     {
       cdef_class cls (get_class ());
 
-      if (! error_state && cls.ok ())
+      if (cls.ok ())
         cls.register_object ();
     }
 }
@@ -1220,7 +1277,7 @@ cdef_object_base::unregister_object (void)
     {
       cdef_class cls (get_class ());
 
-      if (! error_state && cls.ok ())
+      if (cls.ok ())
         cls.unregister_object ();
     }
 }
@@ -1228,7 +1285,7 @@ cdef_object_base::unregister_object (void)
 inline cdef_object_rep*
 cdef_object_base::make_array (void) const
 {
-  cdef_object_rep* r = new cdef_object_array ();
+  cdef_object_rep *r = new cdef_object_array ();
 
   r->set_class (get_class ());
 
@@ -1257,9 +1314,9 @@ private:
     cdef_package_rep (void)
       : cdef_meta_object_rep (), member_count (0) { }
 
-    ~cdef_package_rep (void) { }
+    ~cdef_package_rep (void) = default;
 
-    cdef_object_rep* copy (void) const { return new cdef_package_rep (*this); }
+    cdef_object_rep * copy (void) const { return new cdef_package_rep (*this); }
 
     bool is_package (void) const { return true; }
 
@@ -1302,7 +1359,7 @@ private:
 
     void meta_release (void);
 
-    bool meta_is_postfix_index_handled (char type) const
+    bool meta_accepts_postfix_index (char type) const
     { return (type == '.'); }
 
     octave_value find (const std::string& nm);
@@ -1324,7 +1381,6 @@ private:
     typedef std::map<std::string, cdef_package>::iterator package_iterator;
     typedef std::map<std::string, cdef_package>::const_iterator package_const_iterator;
 
-  private:
     cdef_package_rep (const cdef_package_rep& p)
       : cdef_meta_object_rep (p), full_name (p.full_name),
         class_map (p.class_map), function_map (p.function_map),
@@ -1359,7 +1415,7 @@ public:
 
   cdef_package& operator = (const cdef_package& pack)
   {
-    cdef_object::operator= (pack);
+    cdef_object::operator = (pack);
 
     return *this;
   }
@@ -1386,19 +1442,14 @@ public:
 
   octave_value find (const std::string& nm) { return get_rep ()->find (nm); }
 
-  static const cdef_package& meta (void) { return _meta; }
-
 private:
-  cdef_package_rep* get_rep (void)
+  cdef_package_rep * get_rep (void)
   { return dynamic_cast<cdef_package_rep *> (cdef_object::get_rep ()); }
 
-  const cdef_package_rep* get_rep (void) const
+  const cdef_package_rep * get_rep (void) const
   { return dynamic_cast<const cdef_package_rep *> (cdef_object::get_rep ()); }
 
-private:
-  static cdef_package _meta;
-
-  friend void install_classdef (void);
+  friend void install_classdef (octave::interpreter& interp);
 };
 
 class
@@ -1414,21 +1465,23 @@ public:
   octave_classdef (const octave_classdef& obj)
     : octave_base_value (obj), object (obj.object) { }
 
-  octave_base_value* clone (void) const
+  octave_base_value * clone (void) const
   { return new octave_classdef (object.clone ()); }
 
-  octave_base_value* empty_clone (void) const
+  octave_base_value * empty_clone (void) const
   { return new octave_classdef (object.empty_clone ()); }
 
+  octave_classdef * classdef_object_value (bool = false) { return this; }
+
   cdef_object get_object (void) const { return object; }
 
   cdef_object& get_object_ref (void) { return object; }
 
   bool is_defined (void) const { return true; }
 
-  bool is_map (void) const { return false; }
+  bool isstruct (void) const { return false; }
 
-  bool is_object (void) const { return true; }
+  bool isobject (void) const { return true; }
 
   bool is_classdef_object (void) const { return true; }
 
@@ -1467,16 +1520,25 @@ public:
                   const std::list<octave_value_list>& idx,
                   const octave_value& rhs);
 
+  octave_idx_type numel (const octave_value_list&);
+
   string_vector map_keys (void) const { return object.map_keys (); }
 
   octave_map map_value (void) const { return object.map_value (); }
 
   dim_vector dims (void) const { return object.dims (); }
 
-private:
-  cdef_object object;
+  void set_property (octave_idx_type idx, const std::string& name,
+                     const octave_value& pval)
+  {
+    object.set_property (idx, name, pval);
+  }
 
-private:
+  octave_value
+  get_property (octave_idx_type idx, const std::string& name) const
+  {
+    return object.get_property (idx, name);
+  }
 
 public:
   int type_id (void) const { return t_id; }
@@ -1486,9 +1548,12 @@ public:
   static int static_type_id (void) { return t_id; }
   static std::string static_type_name (void) { return t_name; }
   static std::string static_class_name (void) { return "<unknown>"; }
-  static void register_type (void);
+  static void register_type (octave::type_info&);
 
 private:
+
+  cdef_object object;
+
   static int t_id;
 
   static const std::string t_name;
@@ -1510,173 +1575,138 @@ to_ov (const octave_value& ov)
 inline cdef_object
 to_cdef (const octave_value& val)
 {
-  if (val.type_name () == "object")
-    return dynamic_cast<octave_classdef *> (val.internal_rep ())->get_object ();
-  else
-    {
-      error ("cannot convert `%s' into `object'", val.type_name().c_str ());
-      return cdef_object ();
-    }
+  if (val.type_name () != "object")
+    error ("cannot convert `%s' into `object'", val.type_name().c_str ());
+
+  return dynamic_cast<octave_classdef *> (val.internal_rep ())->get_object ();
 }
 
 inline cdef_object&
 to_cdef_ref (const octave_value& val)
 {
-  static cdef_object empty;
+  if (val.type_name () != "object")
+    error ("cannot convert `%s' into `object'", val.type_name().c_str ());
 
-  if (val.type_name () == "object")
-    return dynamic_cast<octave_classdef *> (val.internal_rep ())->get_object_ref ();
-  else
-    {
-      error ("cannot convert `%s' into `object'", val.type_name().c_str ());
-      return empty;
-    }
+  return dynamic_cast<octave_classdef *> (val.internal_rep ())->get_object_ref ();
 }
 
 inline cdef_object
 to_cdef (const cdef_object& obj)
 { return obj; }
 
-OCTINTERP_API void install_classdef (void);
+OCTINTERP_API void install_classdef (octave::interpreter& interp);
 
 class
 cdef_manager
 {
 public:
 
-  static cdef_class find_class (const std::string& name,
-                                bool error_if_not_found = true,
-                                bool load_if_not_found = true)
-  {
-    if (instance_ok ())
-      return instance->do_find_class (name, error_if_not_found,
-                                      load_if_not_found);
+  cdef_manager (octave::interpreter& interp);
 
-    return cdef_class ();
+  // No copying!
+
+  cdef_manager (const cdef_manager&) = delete;
+
+  cdef_manager& operator = (const cdef_manager&) = delete;
+
+  ~cdef_manager (void) = default;
+
+  cdef_class find_class (const std::string& name, bool error_if_not_found = true,
+                         bool load_if_not_found = true);
+
+  octave_function * find_method_symbol (const std::string& method_name,
+                                        const std::string& class_name);
+
+  cdef_package find_package (const std::string& name,
+                             bool error_if_not_found = true,
+                             bool load_if_not_found = true);
+
+  octave_function * find_package_symbol (const std::string& pack_name);
+
+  void register_class (const cdef_class& cls)
+  {
+    m_all_classes[cls.get_name ()] = cls;
   }
 
-  static octave_function* find_method_symbol (const std::string& method_name,
-      const std::string& class_name)
+  void unregister_class (const cdef_class& cls)
   {
-    if (instance_ok ())
-      return instance->do_find_method_symbol (method_name, class_name);
-
-    return 0;
+    m_all_classes.erase(cls.get_name ());
   }
 
-  static cdef_package find_package (const std::string& name,
-                                    bool error_if_not_found = true,
-                                    bool load_if_not_found = true)
+  void register_package (const cdef_package& pkg)
   {
-    if (instance_ok ())
-      return instance->do_find_package (name, error_if_not_found,
-                                        load_if_not_found);
-
-    return cdef_package ();
+    m_all_packages[pkg.get_name ()] = pkg;
   }
 
-  static octave_function* find_package_symbol (const std::string& pack_name)
+  void unregister_package (const cdef_package& pkg)
   {
-    if (instance_ok ())
-      return instance->do_find_package_symbol (pack_name);
-
-    return 0;
+    m_all_packages.erase (pkg.get_name ());
   }
 
-  static void register_class (const cdef_class& cls)
-  {
-    if (instance_ok ())
-      instance->do_register_class (cls);
-  }
+  const cdef_class& meta_class (void) const { return m_meta_class; }
+  const cdef_class& meta_property (void) const { return m_meta_property; }
+  const cdef_class& meta_method (void) const { return m_meta_method; }
+  const cdef_class& meta_package (void) const { return m_meta_package; }
 
-  static void unregister_class (const cdef_class& cls)
-  {
-    if (instance_ok ())
-      instance->do_unregister_class (cls);
-  }
+  const cdef_package& meta (void) const { return m_meta; }
 
-  static void register_package (const cdef_package& pkg)
-  {
-    if (instance_ok ())
-      instance->do_register_package (pkg);
-  }
+  cdef_class
+  make_class (const std::string& name,
+              const std::list<cdef_class>& super_list = std::list<cdef_class> ());
 
-  static void unregister_package (const cdef_package& pkg)
-  {
-    if (instance_ok ())
-      instance->do_unregister_package (pkg);
-  }
+  cdef_class
+  make_class (const std::string& name, const cdef_class& super);
+
+  cdef_class
+  make_meta_class (const std::string& name, const cdef_class& super);
+
+  cdef_property
+  make_property (const cdef_class& cls, const std::string& name,
+                 const octave_value& get_method = Matrix (),
+                 const std::string& get_access = "public",
+                 const octave_value& set_method = Matrix (),
+                 const std::string& set_access = "public");
+
+  cdef_property
+  make_attribute (const cdef_class& cls, const std::string& name);
+
+  cdef_method
+  make_method (const cdef_class& cls, const std::string& name,
+               const octave_value& fcn,
+               const std::string& m_access = "public",
+               bool is_static = false);
+
+  cdef_method
+  make_method (const cdef_class& cls, const std::string& name,
+               octave_builtin::fcn ff,
+               const std::string& m_access = "public",
+               bool is_static = false);
+
+  cdef_method
+  make_method (const cdef_class& cls, const std::string& name,
+               octave_builtin::meth mm,
+               const std::string& m_access = "public",
+               bool is_static = false);
+
+  cdef_package
+  make_package (const std::string& nm, const std::string& parent = "");
 
 private:
 
-  cdef_manager (void) { }
-
-  cdef_manager (const cdef_manager&);
-
-  cdef_manager& operator = (const cdef_manager&);
-
-  ~cdef_manager (void) { }
-
-  static void create_instance (void);
-
-  static bool instance_ok (void)
-  {
-    bool retval = true;
-
-    if (! instance)
-      create_instance ();
-
-    if (! instance)
-      {
-        ::error ("unable to create cdef_manager!");
-
-        retval = false;
-      }
-
-    return retval;
-  }
-
-  static void cleanup_instance (void)
-  {
-    delete instance;
-
-    instance = 0;
-  }
-
-  cdef_class do_find_class (const std::string& name, bool error_if_not_found,
-                            bool load_if_not_found);
-
-  octave_function* do_find_method_symbol (const std::string& method_name,
-                                          const std::string& class_name);
-
-  cdef_package do_find_package (const std::string& name,
-                                bool error_if_not_found,
-                                bool load_if_not_found);
-
-  octave_function* do_find_package_symbol (const std::string& pack_name);
-
-  void do_register_class (const cdef_class& cls)
-  { all_classes[cls.get_name ()] = cls; }
-
-  void do_unregister_class (const cdef_class& cls)
-  { all_classes.erase(cls.get_name ()); }
-
-  void do_register_package (const cdef_package& pkg)
-  { all_packages[pkg.get_name ()] = pkg; }
-
-  void do_unregister_package (const cdef_package& pkg)
-  { all_packages.erase(pkg.get_name ()); }
-
-private:
-
-  // The single cdef_manager instance
-  static cdef_manager *instance;
+  octave::interpreter& m_interpreter;
 
   // All registered/loaded classes
-  std::map<std::string, cdef_class> all_classes;
+  std::map<std::string, cdef_class> m_all_classes;
 
   // All registered/loaded packages
-  std::map<std::string, cdef_package> all_packages;
+  std::map<std::string, cdef_package> m_all_packages;
+
+  cdef_class m_meta_class;
+  cdef_class m_meta_property;
+  cdef_class m_meta_method;
+  cdef_class m_meta_package;
+
+  cdef_package m_meta;
 };
 
 #endif
