@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 1996-2017 John W. Eaton
+Copyright (C) 1996-2018 John W. Eaton
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -120,30 +120,34 @@ DEFNDCATOP_FN (cm_m, complex_matrix, matrix, complex_array, array, concat)
 DEFNDASSIGNOP_FN (assign, complex_matrix, matrix, complex_array, assign)
 
 void
-install_cm_m_ops (void)
+install_cm_m_ops (octave::type_info& ti)
 {
-  INSTALL_BINOP (op_add, octave_complex_matrix, octave_matrix, add);
-  INSTALL_BINOP (op_sub, octave_complex_matrix, octave_matrix, sub);
-  INSTALL_BINOP (op_mul, octave_complex_matrix, octave_matrix, mul);
-  INSTALL_BINOP (op_div, octave_complex_matrix, octave_matrix, div);
-  INSTALL_BINOP (op_pow, octave_complex_matrix, octave_matrix, pow);
-  INSTALL_BINOP (op_ldiv, octave_complex_matrix, octave_matrix, ldiv);
-  INSTALL_BINOP (op_lt, octave_complex_matrix, octave_matrix, lt);
-  INSTALL_BINOP (op_le, octave_complex_matrix, octave_matrix, le);
-  INSTALL_BINOP (op_eq, octave_complex_matrix, octave_matrix, eq);
-  INSTALL_BINOP (op_ge, octave_complex_matrix, octave_matrix, ge);
-  INSTALL_BINOP (op_gt, octave_complex_matrix, octave_matrix, gt);
-  INSTALL_BINOP (op_ne, octave_complex_matrix, octave_matrix, ne);
-  INSTALL_BINOP (op_el_mul, octave_complex_matrix, octave_matrix, el_mul);
-  INSTALL_BINOP (op_el_div, octave_complex_matrix, octave_matrix, el_div);
-  INSTALL_BINOP (op_el_pow, octave_complex_matrix, octave_matrix, el_pow);
-  INSTALL_BINOP (op_el_ldiv, octave_complex_matrix, octave_matrix, el_ldiv);
-  INSTALL_BINOP (op_el_and, octave_complex_matrix, octave_matrix, el_and);
-  INSTALL_BINOP (op_el_or, octave_complex_matrix, octave_matrix, el_or);
-  INSTALL_BINOP (op_mul_trans, octave_complex_matrix, octave_matrix, mul_trans);
-  INSTALL_BINOP (op_mul_herm, octave_complex_matrix, octave_matrix, mul_trans);
+  INSTALL_BINOP_TI (ti, op_add, octave_complex_matrix, octave_matrix, add);
+  INSTALL_BINOP_TI (ti, op_sub, octave_complex_matrix, octave_matrix, sub);
+  INSTALL_BINOP_TI (ti, op_mul, octave_complex_matrix, octave_matrix, mul);
+  INSTALL_BINOP_TI (ti, op_div, octave_complex_matrix, octave_matrix, div);
+  INSTALL_BINOP_TI (ti, op_pow, octave_complex_matrix, octave_matrix, pow);
+  INSTALL_BINOP_TI (ti, op_ldiv, octave_complex_matrix, octave_matrix, ldiv);
+  INSTALL_BINOP_TI (ti, op_lt, octave_complex_matrix, octave_matrix, lt);
+  INSTALL_BINOP_TI (ti, op_le, octave_complex_matrix, octave_matrix, le);
+  INSTALL_BINOP_TI (ti, op_eq, octave_complex_matrix, octave_matrix, eq);
+  INSTALL_BINOP_TI (ti, op_ge, octave_complex_matrix, octave_matrix, ge);
+  INSTALL_BINOP_TI (ti, op_gt, octave_complex_matrix, octave_matrix, gt);
+  INSTALL_BINOP_TI (ti, op_ne, octave_complex_matrix, octave_matrix, ne);
+  INSTALL_BINOP_TI (ti, op_el_mul, octave_complex_matrix, octave_matrix, el_mul);
+  INSTALL_BINOP_TI (ti, op_el_div, octave_complex_matrix, octave_matrix, el_div);
+  INSTALL_BINOP_TI (ti, op_el_pow, octave_complex_matrix, octave_matrix, el_pow);
+  INSTALL_BINOP_TI (ti, op_el_ldiv, octave_complex_matrix, octave_matrix,
+                    el_ldiv);
+  INSTALL_BINOP_TI (ti, op_el_and, octave_complex_matrix, octave_matrix, el_and);
+  INSTALL_BINOP_TI (ti, op_el_or, octave_complex_matrix, octave_matrix, el_or);
+  INSTALL_BINOP_TI (ti, op_mul_trans, octave_complex_matrix, octave_matrix,
+                    mul_trans);
+  INSTALL_BINOP_TI (ti, op_mul_herm, octave_complex_matrix, octave_matrix,
+                    mul_trans);
 
-  INSTALL_CATOP (octave_complex_matrix, octave_matrix, cm_m);
+  INSTALL_CATOP_TI (ti, octave_complex_matrix, octave_matrix, cm_m);
 
-  INSTALL_ASSIGNOP (op_asn_eq, octave_complex_matrix, octave_matrix, assign);
+  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_complex_matrix, octave_matrix,
+                       assign);
 }
