@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 1996-2017 John W. Eaton
+Copyright (C) 1996-2018 John W. Eaton
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -141,39 +141,43 @@ DEFCONV (complex_matrix_conv, matrix, complex_matrix)
 }
 
 void
-install_m_cm_ops (void)
+install_m_cm_ops (octave::type_info& ti)
 {
-  INSTALL_BINOP (op_add, octave_matrix, octave_complex_matrix, add);
-  INSTALL_BINOP (op_sub, octave_matrix, octave_complex_matrix, sub);
-  INSTALL_BINOP (op_mul, octave_matrix, octave_complex_matrix, mul);
-  INSTALL_BINOP (op_div, octave_matrix, octave_complex_matrix, div);
-  INSTALL_BINOP (op_pow, octave_matrix, octave_complex_matrix, pow);
-  INSTALL_BINOP (op_ldiv, octave_matrix, octave_complex_matrix, ldiv);
-  INSTALL_BINOP (op_lt, octave_matrix, octave_complex_matrix, lt);
-  INSTALL_BINOP (op_le, octave_matrix, octave_complex_matrix, le);
-  INSTALL_BINOP (op_eq, octave_matrix, octave_complex_matrix, eq);
-  INSTALL_BINOP (op_ge, octave_matrix, octave_complex_matrix, ge);
-  INSTALL_BINOP (op_gt, octave_matrix, octave_complex_matrix, gt);
-  INSTALL_BINOP (op_ne, octave_matrix, octave_complex_matrix, ne);
-  INSTALL_BINOP (op_el_mul, octave_matrix, octave_complex_matrix, el_mul);
-  INSTALL_BINOP (op_el_div, octave_matrix, octave_complex_matrix, el_div);
-  INSTALL_BINOP (op_el_pow, octave_matrix, octave_complex_matrix, el_pow);
-  INSTALL_BINOP (op_el_ldiv, octave_matrix, octave_complex_matrix, el_ldiv);
-  INSTALL_BINOP (op_el_and, octave_matrix, octave_complex_matrix, el_and);
-  INSTALL_BINOP (op_el_or, octave_matrix, octave_complex_matrix, el_or);
-  INSTALL_BINOP (op_trans_mul, octave_matrix, octave_complex_matrix, trans_mul);
-  INSTALL_BINOP (op_herm_mul, octave_matrix, octave_complex_matrix, trans_mul);
-  INSTALL_BINOP (op_trans_ldiv, octave_matrix, octave_complex_matrix,
-                 trans_ldiv);
-  INSTALL_BINOP (op_herm_ldiv, octave_matrix, octave_complex_matrix,
-                 trans_ldiv);
+  INSTALL_BINOP_TI (ti, op_add, octave_matrix, octave_complex_matrix, add);
+  INSTALL_BINOP_TI (ti, op_sub, octave_matrix, octave_complex_matrix, sub);
+  INSTALL_BINOP_TI (ti, op_mul, octave_matrix, octave_complex_matrix, mul);
+  INSTALL_BINOP_TI (ti, op_div, octave_matrix, octave_complex_matrix, div);
+  INSTALL_BINOP_TI (ti, op_pow, octave_matrix, octave_complex_matrix, pow);
+  INSTALL_BINOP_TI (ti, op_ldiv, octave_matrix, octave_complex_matrix, ldiv);
+  INSTALL_BINOP_TI (ti, op_lt, octave_matrix, octave_complex_matrix, lt);
+  INSTALL_BINOP_TI (ti, op_le, octave_matrix, octave_complex_matrix, le);
+  INSTALL_BINOP_TI (ti, op_eq, octave_matrix, octave_complex_matrix, eq);
+  INSTALL_BINOP_TI (ti, op_ge, octave_matrix, octave_complex_matrix, ge);
+  INSTALL_BINOP_TI (ti, op_gt, octave_matrix, octave_complex_matrix, gt);
+  INSTALL_BINOP_TI (ti, op_ne, octave_matrix, octave_complex_matrix, ne);
+  INSTALL_BINOP_TI (ti, op_el_mul, octave_matrix, octave_complex_matrix, el_mul);
+  INSTALL_BINOP_TI (ti, op_el_div, octave_matrix, octave_complex_matrix, el_div);
+  INSTALL_BINOP_TI (ti, op_el_pow, octave_matrix, octave_complex_matrix, el_pow);
+  INSTALL_BINOP_TI (ti, op_el_ldiv, octave_matrix, octave_complex_matrix,
+                    el_ldiv);
+  INSTALL_BINOP_TI (ti, op_el_and, octave_matrix, octave_complex_matrix, el_and);
+  INSTALL_BINOP_TI (ti, op_el_or, octave_matrix, octave_complex_matrix, el_or);
+  INSTALL_BINOP_TI (ti, op_trans_mul, octave_matrix, octave_complex_matrix,
+                    trans_mul);
+  INSTALL_BINOP_TI (ti, op_herm_mul, octave_matrix, octave_complex_matrix,
+                    trans_mul);
+  INSTALL_BINOP_TI (ti, op_trans_ldiv, octave_matrix, octave_complex_matrix,
+                    trans_ldiv);
+  INSTALL_BINOP_TI (ti, op_herm_ldiv, octave_matrix, octave_complex_matrix,
+                    trans_ldiv);
 
-  INSTALL_CATOP (octave_matrix, octave_complex_matrix, m_cm);
+  INSTALL_CATOP_TI (ti, octave_matrix, octave_complex_matrix, m_cm);
 
-  INSTALL_ASSIGNCONV (octave_matrix, octave_complex_matrix,
-                      octave_complex_matrix);
-  INSTALL_ASSIGNCONV (octave_float_matrix, octave_complex_matrix,
-                      octave_float_complex_matrix);
+  INSTALL_ASSIGNCONV_TI (ti, octave_matrix, octave_complex_matrix,
+                         octave_complex_matrix);
+  INSTALL_ASSIGNCONV_TI (ti, octave_float_matrix, octave_complex_matrix,
+                         octave_float_complex_matrix);
 
-  INSTALL_WIDENOP (octave_matrix, octave_complex_matrix, complex_matrix_conv);
+  INSTALL_WIDENOP_TI (ti, octave_matrix, octave_complex_matrix,
+                      complex_matrix_conv);
 }
