@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 1996-2017 John W. Eaton
+Copyright (C) 1996-2018 John W. Eaton
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -53,7 +53,7 @@ namespace octave
         {
           close ();
 
-          std::string fullname = octave::sys::file_ops::tilde_expand (name);
+          std::string fullname = sys::file_ops::tilde_expand (name);
 
           dir = octave_opendir_wrapper (fullname.c_str ());
 
@@ -63,7 +63,7 @@ namespace octave
       else
         errmsg = "dir_entry::open: empty filename";
 
-      return dir != 0;
+      return dir != nullptr;
     }
 
     string_vector
@@ -95,7 +95,7 @@ namespace octave
         {
           retval = (octave_closedir_wrapper (dir) == 0);
 
-          dir = 0;
+          dir = nullptr;
         }
 
       return retval;
