@@ -825,23 +825,23 @@ INSTANTIATE_INTTYPE (uint64_t);
 %!assert ((int64 (2**62)+1)**1, int64 (2**62)+1)
 %!assert ((int64 (2**30)+1)**2, int64 (2**60+2**31) + 1)
 
-%!assert (uint8 (char (128)), uint8 (128))
-%!assert (uint8 (char (255)), uint8 (255))
-%!assert (int8 (char (128)), int8 (128))
-%!assert (int8 (char (255)), int8 (255))
+%!assert <54382> (uint8 (char (128)), uint8 (128))
+%!assert <54382> (uint8 (char (255)), uint8 (255))
+%!assert <54382> (int8 (char (128)), int8 (128))
+%!assert <54382> (int8 (char (255)), int8 (255))
 
-%!assert (uint16 (char (128)), uint16 (128))
-%!assert (uint16 (char (255)), uint16 (255))
-%!assert (int16 (char (128)), int16 (128))
-%!assert (int16 (char (255)), int16 (255))
+%!assert <54382> (uint16 (char (128)), uint16 (128))
+%!assert <54382> (uint16 (char (255)), uint16 (255))
+%!assert <54382> (int16 (char (128)), int16 (128))
+%!assert <54382> (int16 (char (255)), int16 (255))
 
-%!assert (uint32 (char (128)), uint32 (128))
-%!assert (uint32 (char (255)), uint32 (255))
-%!assert (int32 (char (128)), int32 (128))
-%!assert (int32 (char (255)), int32 (255))
+%!assert <54382> (uint32 (char (128)), uint32 (128))
+%!assert <54382> (uint32 (char (255)), uint32 (255))
+%!assert <54382> (int32 (char (128)), int32 (128))
+%!assert <54382> (int32 (char (255)), int32 (255))
 
-%!assert (uint64 (char (128)), uint64 (128))
-%!assert (uint64 (char (255)), uint64 (255))
-%!assert (int64 (char (128)), int64 (128))
-%!assert (int64 (char (255)), int64 (255))
+%!assert <54382> (uint64 (char (128)), uint64 (128))
+%!assert <54382> (uint64 (char (255)), uint64 (255))
+%!assert <54382> (int64 (char (128)), int64 (128))
+%!assert <54382> (int64 (char (255)), int64 (255))
 */
