@@ -1,20 +1,20 @@
-## Copyright (C) 2006-2016 Keith Goodman
+## Copyright (C) 2006-2018 Keith Goodman
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} mkoctfile [-options] file @dots{}
@@ -54,12 +54,12 @@
 ## Add the run-time path to the link command.
 ##
 ## @item @nospell{-Wl,@dots{}}
-## Pass flags though the linker like @nospell{"-Wl,-rpath=@dots{}"}.
+## Pass options to the linker like @nospell{"-Wl,-rpath=@dots{}"}.
 ## The quotes are needed since commas are interpreted as command
 ## separators.
 ##
 ## @item -W@dots{}
-## Pass flags though the compiler like @nospell{"-Wa,OPTION"}.
+## Pass options to the assembler like @nospell{"-Wa,OPTION"}.
 ##
 ## @item -c
 ## Compile but do not link.
@@ -172,4 +172,3 @@ function [output, status] = mkoctfile (varargin)
   endif
 
 endfunction
-
