@@ -1,24 +1,24 @@
-## Copyright (C) 1997-2015 Vincent Cautaerts
+## Copyright (C) 1997-2018 Vincent Cautaerts
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn  {Function File} {} ifftshift (@var{x})
-## @deftypefnx {Function File} {} ifftshift (@var{x}, @var{dim})
+## @deftypefn  {} {} ifftshift (@var{x})
+## @deftypefnx {} {} ifftshift (@var{x}, @var{dim})
 ## Undo the action of the @code{fftshift} function.
 ##
 ## For even length @var{x}, @code{fftshift} is its own inverse, but odd lengths
@@ -34,15 +34,17 @@
 
 function retval = ifftshift (x, dim)
 
-  retval = 0;
-
   if (nargin != 1 && nargin != 2)
     print_usage ();
   endif
 
+  if (! (isnumeric (x) || islogical (x) || ischar (x)))
+    error ("ifftshift: X must be a vector or matrix");
+  endif
+
   if (nargin == 2)
-    if (! isscalar (dim))
-      error ("ifftshift: dimension must be an integer scalar");
+    if (! (isscalar (dim) && dim > 0 && dim == fix (dim)))
+      error ("ifftshift: dimension DIM must be a positive integer");
     endif
     nd = ndims (x);
     sz = size (x);
@@ -55,7 +57,7 @@ function retval = ifftshift (x, dim)
       xl = length (x);
       xx = floor (xl/2);
       retval = x([xx+1:xl, 1:xx]);
-    elseif (ismatrix (x))
+    else
       nd = ndims (x);
       sz = size (x);
       sz2 = floor (sz ./ 2);
@@ -64,8 +66,6 @@ function retval = ifftshift (x, dim)
         idx{i} = [sz2(i)+1:sz(i), 1:sz2(i)];
       endfor
       retval = x(idx{:});
-    else
-      error ("ifftshift: expecting vector or matrix argument");
     endif
   endif
 
@@ -117,3 +117,23 @@ endfunction
 %! assert (y, [[2 3 0 1];[4 6 0 2];[7 10 1 4];[9 13 1 5]]);
 %! assert (ifftshift (y,2), x);
 
+%!test
+%! x = "efgabcd";
+%! y = ifftshift (x);
+%! assert (y, "abcdefg");
+%! assert (ifftshift (y), "defgabc");
+
+## Test N-dimensional input
+%!test <*45207>
+%! x = [0:3];
+%! x = x + x' + reshape (x, [1 1 4]);
+%! y1 = [4 5 2 3; 5 6 3 4; 2 3 0 1; 3 4 1 2];
+%! y = ifftshift (x);
+%! assert (y, reshape ([y1 + 2, y1 + 3, y1, y1 + 1], [4 4 4]));
+%! assert (ifftshift (y), x);
+
+## Test input validation
+%!error ifftshift ()
+%!error ifftshift (1, 2, 3)
+%!error ifftshift (0:3, -1)
+%!error ifftshift (0:3, 0:3)
